/*  Sirikata
 *  MotionPath.hpp
 *
 *  Copyright (c) 2009, Ewen Cheslack-Postava
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of Sirikata nor the names of its contributors may
 *    be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _SIRIKATA_MOTION_PATH_HPP_
#define _SIRIKATA_MOTION_PATH_HPP_

#include <sirikata/core/util/Platform.hpp>
#include "MotionVector.hpp"

namespace Sirikata {

/** Base class for MotionPaths. All motion paths must implement one method
 *  that allows random access into the update list, given the next update
 *  that will occur after a specified time.
 */
class MotionPath {
public:
    virtual ~MotionPath() {}

    /** The initial motion vector for the object. */
    virtual const TimedMotionVector3f initial() const = 0;
    /** Given a current time, return the next update that will occur. Returns
     *  NULL if there is not another update.
     */
    virtual const TimedMotionVector3f* nextUpdate(const Time& curtime) const = 0;
    /** Get the value at the specified time. */
    virtual const TimedMotionVector3f at(const Time& t) const = 0;
}; // class MotionPath

} // namespace Sirikata

#endif //_SIRIKATA_MOTION_PATH_HPP_
